@@ -55,168 +55,6 @@ static const char *const counter_str[] = {
 	[PAI_KERNEL]     = "pai_kernel"
 };
 
-/* Strings for the pai counter details.  Note that this is 0-based
- * while PoP is 1-based.
- */
-static const char *const pai_str[] = {
-	[  0] = "KM DES",
-	[  1] = "KM 2key TDES",
-	[  2] = "KM TDES",
-	[  3] = "KM DES protected key",
-	[  4] = "KM 2key TDES protected key",
-	[  5] = "KM TDES protected key",
-	[  6] = "KM AES 128bit",
-	[  7] = "KM AES 192bit",
-	[  8] = "KM AES 256bit",
-	[  9] = "KM AES 128bit protected key",
-	[ 10] = "KM AES 192bit protected key",
-	[ 11] = "KM AES 256bit protected key",
-	[ 12] = "KM AES-XTS 128bit",
-	[ 13] = "KM AES-XTS 256bit",
-	[ 14] = "KM AES-XTS 128bit protected key",
-	[ 15] = "KM AES-XTS 256bit protected key",
-	[ 16] = "KMC DES",
-	[ 17] = "KMC 2key TDES",
-	[ 18] = "KMC TDES",
-	[ 19] = "KMC DES protected key",
-	[ 20] = "KMC 2key TDES protected key",
-	[ 21] = "KMC TDES protected key",
-	[ 22] = "KMC AES 128bit",
-	[ 23] = "KMC AES 192bit",
-	[ 24] = "KMC AES 256bit",
-	[ 25] = "KMC AES 128bit protected key",
-	[ 26] = "KMC AES 192bit protected key",
-	[ 27] = "KMC AES 256bit protected key",
-	[ 28] = "KMC PRNG",
-	[ 29] = "KMA AES 128bit",
-	[ 30] = "KMA AES 192bit",
-	[ 31] = "KMA AES 256bit",
-	[ 32] = "KMA AES 128bit protected key",
-	[ 33] = "KMA AES 192bit protected key",
-	[ 34] = "KMA AES 256bit protected key",
-	[ 35] = "KMF DES",
-	[ 36] = "KMF 2key TDES",
-	[ 37] = "KMF TDES",
-	[ 38] = "KMF DES protected key",
-	[ 39] = "KMF 2key TDES protected key",
-	[ 40] = "KMF TDES protected key",
-	[ 41] = "KMF AES 128bit",
-	[ 42] = "KMF AES 192bit",
-	[ 43] = "KMF AES 256bit",
-	[ 44] = "KMF AES 128bit protected key",
-	[ 45] = "KMF AES 192bit protected key",
-	[ 46] = "KMF AES 256bit protected key",
-	[ 47] = "KMCTR DES",
-	[ 48] = "KMCTR 2key TDES",
-	[ 49] = "KMCTR TDES",
-	[ 50] = "KMCTR DES protected key",
-	[ 51] = "KMCTR 2key TDES protected key",
-	[ 52] = "KMCTR TDES protected key",
-	[ 53] = "KMCTR AES 128bit",
-	[ 54] = "KMCTR AES 192bit",
-	[ 55] = "KMCTR AES 256bit",
-	[ 56] = "KMCTR AES 128bit protected key",
-	[ 57] = "KMCTR AES 192bit protected key",
-	[ 58] = "KMCTR AES 256bit protected key",
-	[ 59] = "KMO DES",
-	[ 60] = "KMO 2key TDES",
-	[ 61] = "KMO TDES",
-	[ 62] = "KMO DES protected key",
-	[ 63] = "KMO 2key TDES protected key",
-	[ 64] = "KMO TDES protected key",
-	[ 65] = "KMO AES 128bit",
-	[ 66] = "KMO AES 192bit",
-	[ 67] = "KMO AES 256bit",
-	[ 68] = "KMO AES 128bit protected key",
-	[ 69] = "KMO AES 192bit protected key",
-	[ 70] = "KMO AES 256bit protected key",
-	[ 71] = "KIMD SHA1",
-	[ 72] = "KIMD SHA256",
-	[ 73] = "KIMD SHA512",
-	[ 74] = "KIMD SHA3-224",
-	[ 75] = "KIMD SHA3-256",
-	[ 76] = "KIMD SHA3-384",
-	[ 77] = "KIMD SHA3-512",
-	[ 78] = "KIMD SHAKE 128",
-	[ 79] = "KIMD SHAKE 256",
-	[ 80] = "KIMD GHASH",
-	[ 81] = "KLMD SHA1",
-	[ 82] = "KLMD SHA256",
-	[ 83] = "KLMD SHA512",
-	[ 84] = "KLMD SHA3-224",
-	[ 85] = "KLMD SHA3-256",
-	[ 86] = "KLMD SHA3-384",
-	[ 87] = "KLMD SHA3-512",
-	[ 88] = "KLMD SHAKE 128",
-	[ 89] = "KLMD SHAKE 256",
-	[ 90] = "KMAC DES",
-	[ 91] = "KMAC 2key TDES",
-	[ 92] = "KMAC TDES",
-	[ 93] = "KMAC DES protected key",
-	[ 94] = "KMAC 2key TDES protected key",
-	[ 95] = "KMAC TDES protected key",
-	[ 96] = "KMAC AES 128bit",
-	[ 97] = "KMAC AES 192bit",
-	[ 98] = "KMAC AES 256bit",
-	[ 99] = "KMAC AES 128bit protected key",
-	[100] = "KMAC AES 192bit protected key",
-	[101] = "KMAC AES 256bit protected key",
-	[102] = "PCC Last Block CMAC DES",
-	[103] = "PCC Last Block CMAC 2key TDES",
-	[104] = "PCC Last Block CMAC TDES",
-	[105] = "PCC Last Block CMAC DES protected key",
-	[106] = "PCC Last Block CMAC 2key TDES protected key",
-	[107] = "PCC Last Block CMAC TDES protected key",
-	[108] = "PCC Last Block CMAC AES 128bit",
-	[109] = "PCC Last Block CMAC AES 192bit",
-	[110] = "PCC Last Block CMAC AES 256bit",
-	[111] = "PCC Last Block CMAC AES 128bit protected key",
-	[112] = "PCC Last Block CMAC AES 192bit protected key",
-	[113] = "PCC Last Block CMAC AES 256bit protected key",
-	[114] = "PCC XTS Parameter AES 128bit",
-	[115] = "PCC XTS Parameter AES 256bit",
-	[116] = "PCC XTS Parameter AES 128bit protected key",
-	[117] = "PCC XTS Parameter AES 256bit protected key",
-	[118] = "PCC Scalar Mult P256",
-	[119] = "PCC Scalar Mult P384",
-	[120] = "PCC Scalar Mult P521",
-	[121] = "PCC Scalar Mult Ed25519",
-	[122] = "PCC Scalar Mult Ed448",
-	[123] = "PCC Scalar Mult X25519",
-	[124] = "PCC Scalar Mult X448",
-	[125] = "PRNO SHA512 DRNG",
-	[126] = "PRNO TRNG Query Ratio",
-	[127] = "PRNO TRNG",
-	[128] = "KDSA ECDSA Verify P256",
-	[129] = "KDSA ECDSA Verify P384",
-	[130] = "KDSA ECDSA Verify P521",
-	[131] = "KDSA ECDSA Sign P256",
-	[132] = "KDSA ECDSA Sign P384",
-	[133] = "KDSA ECDSA Sign P521",
-	[134] = "KDSA ECDSA Sign P256 protected key",
-	[135] = "KDSA ECDSA Sign P384 protected key",
-	[136] = "KDSA ECDSA Sign P521 protected key",
-	[137] = "KDSA EdDSA Verify Ed25519",
-	[138] = "KDSA EdDSA Verify Ed448",
-	[139] = "KDSA EdDSA Sign Ed25519",
-	[140] = "KDSA EdDSA Sign Ed448",
-	[141] = "KDSA EdDSA Sign Ed25519 protected key",
-	[142] = "KDSA EdDSA Sign Ed448 protected key",
-	[143] = "PCKMO DES",
-	[144] = "PCKMO 2key TDES",
-	[145] = "PCKMO TDES",
-	[146] = "PCKMO AES 128bit",
-	[147] = "PCKMO AES 192bit",
-	[148] = "PCKMO AES 256bit",
-	[149] = "PCKMO ECC P256",
-	[150] = "PCKMO ECC P384",
-	[151] = "PCKMO ECC P521",
-	[152] = "PCKMO ECC Ed25519",
-	[153] = "PCKMO ECC Ed448",
-	[154] = "Reserved 1",
-	[155] = "Reserved 2"
-};
-
 
 static int paiprintnonzero;
 
@@ -275,7 +113,8 @@ static void json_print_virtual_counter_answer(int s, int ctr,
 					      int state, uint64_t value)
 {
 	int paictr = 0, paistate = 0, ec;
-	uint64_t i, paivalue = 0, maxnum;
+	uint64_t i, paivalue = 0;
+	unsigned int maxnum;
 	const char *space;
 
 	switch (ctr) {
@@ -288,11 +127,11 @@ static void json_print_virtual_counter_answer(int s, int ctr,
 			printf("\"value\":%d}", !!value);
 		return;
 	case PAI_USER:
-		maxnum = NUM_PAI_USER;
+		maxnum = get_num_user_space_ctrs();
 		space = "user";
 		break;
 	case PAI_KERNEL:
-		maxnum = NUM_PAI_KERNEL;
+		maxnum = MAX_NUM_PAI;
 		space = "kernel";
 		break;
 	default:
@@ -304,7 +143,7 @@ static void json_print_virtual_counter_answer(int s, int ctr,
 		return;
 	if (value > maxnum) {
 		eprint("Incompatible versions detected!\n");
-		eprint("Expected %"PRIu64" counter space for %s, but got %"PRIu64"\n",
+		eprint("Expected %lu counter space for %s, but got %lu\n",
 			maxnum, space, value);
 		exit(EXIT_FAILURE);
 	}
@@ -315,18 +154,18 @@ static void json_print_virtual_counter_answer(int s, int ctr,
 			/* No more data for this virtual event after error. */
 			return;
 		}
-		if (paictr > NUM_PAI_KERNEL) {
+		if (paictr > MAX_NUM_PAI) {
 			eprint("Pai counter number too big: %d\n", paictr);
 		} else {
 			printjsonsep();
 			printf("{\"counter\":\"%s\",\"space\":\"%s\",\"counterid\":%d,",
-				pai_str[paictr], space, paictr + 1);
+				get_ctr_name(paictr), space, paictr + 1);
 			if (paistate < 0) {
 				printf("\"error\":%d}", paistate);
 				/* Protocol does not send further counters. */
 				return;
 			}
-			printf("\"value\":%"PRIu64"}", paivalue);
+			printf("\"value\":%lu}", paivalue);
 		}
 	}
 }
@@ -341,7 +180,8 @@ static void print_virtual_counter_answer(int s,
 		[UNSUPPORTED] = "unsupported"
 	};
 	int paictr = 0, paistate = 0, ec;
-	uint64_t i, paivalue = 0, maxnum;
+	uint64_t i, paivalue = 0;
+	unsigned int maxnum;
 	const char *ctrstr;
 
 	switch (ctr) {
@@ -350,11 +190,11 @@ static void print_virtual_counter_answer(int s,
 			printf(" hotplug detected\n");
 		return;
 	case PAI_USER:
-		maxnum = NUM_PAI_USER;
+		maxnum = get_num_user_space_ctrs();
 		ctrstr = "pai_user";
 		break;
 	case PAI_KERNEL:
-		maxnum = NUM_PAI_KERNEL;
+		maxnum = MAX_NUM_PAI;
 		ctrstr = "pai_kernel";
 		break;
 	default:
@@ -372,7 +212,7 @@ static void print_virtual_counter_answer(int s,
 		return;
 	if (value > maxnum) {
 		eprint("Incompatible versions detected!\n");
-		eprint("Expected %"PRIu64" counters for %s, but got %"PRIu64"\n",
+		eprint("Expected %lu counters for %s, but got %lu\n",
 		       maxnum, ctrstr, value);
 		exit(EXIT_FAILURE);
 	}
@@ -383,10 +223,11 @@ static void print_virtual_counter_answer(int s,
 			/* No more data for this virtual event after error. */
 			return;
 		}
-		if (paictr > NUM_PAI_KERNEL)
+		if (paictr > MAX_NUM_PAI)
 			eprint("Pai counter number too big: %d\n", paictr);
 		else if (!paiprintnonzero || paivalue > 0)
-			printf("  %-45s: %"PRIu64"\n", pai_str[paictr], paivalue);
+			printf("  (%3d) %-45s: %lu\n", paictr + 1,
+			       get_ctr_name(paictr), paivalue);
 	}
 }
 
@@ -402,7 +243,7 @@ static void print_answer(int s, int ctr, int state, uint64_t value)
 	else if (state == UNSUPPORTED)
 		printf(" %s counter: unsupported\n", counter_str[ctr]);
 	else
-		printf(" %s counter: %"PRIu64"\n", counter_str[ctr], value);
+		printf(" %s counter: %lu\n", counter_str[ctr], value);
 }
 
 
@@ -417,7 +258,7 @@ static void json_print_answer(int s, int ctr, int state, uint64_t value)
 	} else if (state == ENABLED) {
 		printjsonsep();
 		printf("{\"counter\":\"%s\",", counter_str[ctr]);
-		printf("\"value\":%"PRIu64"}", value);
+		printf("\"value\":%lu}", value);
 	}
 }
 
