@@ -1,14 +1,12 @@
 /*
- * File...........: s390-tools/fdasd/vtoc.c
- * Author(s)......: Volker Sameske <sameske@de.ibm.com>
- * Copyright IBM Corp. 2001,2007
+ * Copyright IBM Corp. 2001, 2017
+ *
+ * s390-tools is free software; you can redistribute it and/or modify
+ * it under the terms of the MIT license. See LICENSE for details.
  *
- * History of changes (starts March 2002)
- * 2002-03-12 removed dependency to kernel headers
- * 2003-01-30 changed DS4VTOCI from 0x20 to 0xa0
  */
 
-#include "vtoc.h"
+#include "lib/vtoc.h"
 
 static unsigned char EBCtoASC[256] =
 {
@@ -146,30 +144,25 @@ static char buffer[85];
  */
 static void vtoc_error(enum failure why, char *s1, char *s2) 
 {
-        char    error[LINE_LENGTH];
-
 	switch (why) {
 	case unable_to_open:
-	        sprintf(error, "%s opening device '%s' failed.\n%s\n",
+	        fprintf(stderr, "\n%s opening device '%s' failed.\n%s\n",
 			VTOC_ERROR, s1, s2);
 		break;
 	case unable_to_seek:
-	        sprintf(error, "%s seeking device '%s' failed.\n%s\n",
+	        fprintf(stderr, "\n%s seeking device '%s' failed.\n%s\n",
 			VTOC_ERROR, s1, s2);
 		break;
 	case unable_to_write:
-	        sprintf(error, "%s writing to device '%s' failed,\n%s\n",
+	        fprintf(stderr, "\n%s writing to device '%s' failed,\n%s\n",
 			VTOC_ERROR, s1, s2);
 		break;
 	case unable_to_read:
-	        sprintf(error, "%s reading from device '%s' failed.\n%s\n",
+	        fprintf(stderr, "\n%s reading from device '%s' failed.\n%s\n",
 			VTOC_ERROR, s1, s2);
 		break;
-	default: sprintf(error, "Fatal error\n");
+	default: fprintf(stderr, "\nFatal error\n");
 	}
-
-	fputc('\n', stderr);
-	fputs(error, stderr);
 	exit(1);
 }
 
@@ -218,19 +211,37 @@ void vtoc_set_extent (extent_t *ext, u_int8_t typeind, u_int8_t seqno,
 /*
  *
  */
-void vtoc_set_cchh (cchh_t *addr, u_int16_t cc, u_int16_t hh) 
+void vtoc_set_cchh (cchh_t *addr, u_int32_t cc, u_int16_t hh)
 {
-        addr->cc = cc;
-	addr->hh = hh;
+	addr->cc = (u_int16_t) cc;
+	addr->hh = cc >> 16;
+	addr->hh <<= 4;
+	addr->hh |= hh;
 }
 
+u_int32_t vtoc_get_cyl_from_cchh(cchh_t *addr)
+{
+        u_int32_t cyl;
+
+        /*decode cylinder for large volumes */
+	cyl = addr->hh & 0xFFF0;
+	cyl <<= 12;
+	cyl |= addr->cc;
+	return cyl;
+}
+
+u_int16_t vtoc_get_head_from_cchh(cchh_t *addr)
+{
+	/* decode heads for large volumes */
+	return addr->hh & 0x000F;
+}
 
 /*
  *
  */
-static void vtoc_set_ttr (ttr_t *addr, u_int16_t tt, u_int8_t r) 
+static void vtoc_set_ttr (ttr_t *addr, u_int16_t tt, u_int8_t r)
 {
-        addr->tt = tt;
+	addr->tt = tt;
 	addr->r  = r;
 }
 
@@ -238,13 +249,58 @@ static void vtoc_set_ttr (ttr_t *addr, u_int16_t tt, u_int8_t r)
 /*
  *
  */
-void vtoc_set_cchhb (cchhb_t *addr, u_int16_t cc, u_int16_t hh, u_int8_t b) 
+void vtoc_set_cchhb (cchhb_t *addr, u_int32_t cc, u_int16_t hh, u_int8_t b)
 {
-        addr->cc = cc;
-        addr->hh = hh;
-        addr->b  = b;
+	addr->cc = (u_int16_t) cc;
+	addr->hh = cc >> 16;
+	addr->hh <<= 4;
+	addr->hh |= hh;
+	addr->b  = b;
 }
 
+u_int32_t vtoc_get_cyl_from_cchhb(cchhb_t *addr)
+{
+        u_int32_t cyl;
+
+        /* decode cylinder for large volumes */
+        cyl = addr->hh & 0xFFF0;
+        cyl <<= 12;
+        cyl |= addr->cc;
+	return cyl;
+}
+
+u_int16_t vtoc_get_head_from_cchhb(cchhb_t *addr)
+{
+	/* decode heads for large volumes */
+	return addr->hh & 0x000F;
+}
+
+/*
+ * some functions to convert cyl-cyl-head-head addresses to
+ * block or track numbers
+ * Note: Record zero is special, so first block on a track is
+ * in record 1!
+ */
+u_int64_t cchhb2blk(cchhb_t *p, struct hd_geometry *geo)
+{
+        return (u_int64_t) vtoc_get_cyl_from_cchhb(p) *
+		geo->heads * geo->sectors +
+		vtoc_get_head_from_cchhb(p) * geo->sectors +
+		p->b;
+}
+
+u_int64_t cchh2blk (cchh_t *p, struct hd_geometry *geo) {
+
+        return (u_int64_t) vtoc_get_cyl_from_cchh(p) *
+		geo->heads * geo->sectors +
+                vtoc_get_head_from_cchh(p) * geo->sectors;
+}
+
+u_int32_t cchh2trk (cchh_t *p, struct hd_geometry *geo) {
+
+	return  vtoc_get_cyl_from_cchh(p) * geo->heads +
+		vtoc_get_head_from_cchh(p);
+}
 
 /*
  *
@@ -263,7 +319,7 @@ void vtoc_volume_label_init (volume_label_t *vlabel)
 {
 	sprintf(buffer, "%84s", " ");
 	vtoc_ebcdic_enc(buffer, buffer, 84);	
-	strncpy(vlabel->volkey, buffer, 84);
+	memcpy(vlabel, buffer, 84);
 }
 
 
@@ -342,7 +398,7 @@ void vtoc_volume_label_set_volser (volume_label_t *vlabel, char *volser)
 
 	strcpy(s, "      ");
 	vtoc_ebcdic_enc(s, s, VOLSER_LENGTH);
-	strncpy(vlabel->volid, s, VOLSER_LENGTH);
+	memcpy(vlabel->volid, s, VOLSER_LENGTH);
 
 	if (i > VOLSER_LENGTH) i = VOLSER_LENGTH;
 
@@ -481,7 +537,8 @@ vtoc_write_label (char *device,
 		  format1_label_t *f1, 
 		  format4_label_t *f4,
 		  format5_label_t *f5,
-		  format7_label_t *f7)
+		  format7_label_t *f7,
+		  format9_label_t *f9)
 {
 	int f,t;
 
@@ -542,6 +599,17 @@ vtoc_write_label (char *device,
 		}
 	}
 
+	if (f9 != NULL)
+	{
+		t = sizeof(format9_label_t);
+	        if (write(f, f9, t) != t)
+		{
+		        close(f);
+			vtoc_error(unable_to_write, device,
+				   "Could not write VTOC FMT9 DSCB.");
+		}
+	}
+
         close(f);
 }
 
@@ -552,8 +620,8 @@ vtoc_write_label (char *device,
 void 
 vtoc_init_format4_label (
 	format4_label_t    *f4,
-	unsigned int usable_partitions,
-	unsigned int cylinders,
+	unsigned int compat_cylinders,
+	unsigned int real_cylinders,
 	unsigned int tracks,
 	unsigned int blocks,
 	unsigned int blksize,
@@ -578,7 +646,7 @@ vtoc_init_format4_label (
         f4->DS4DEVAC = 0x00;
 
         /* -- begin f4->DS4DEVCT -- */
-        f4->DS4DEVCT.DS4DSCYL = cylinders;
+        f4->DS4DEVCT.DS4DSCYL = compat_cylinders;
         f4->DS4DEVCT.DS4DSTRK = tracks;
 
 	switch (dev_type)
@@ -619,7 +687,11 @@ vtoc_init_format4_label (
 	bzero(f4->res2, sizeof(f4->res2));
 	f4->DS4EFLVL = 0x00;
 	bzero(&f4->DS4EFPTR, sizeof(f4->DS4EFPTR));
-	bzero(f4->res3, sizeof(f4->res3));
+	bzero(&f4->res3, sizeof(f4->res3));
+	f4->DS4DCYL = real_cylinders;
+	bzero(f4->res4, sizeof(f4->res4));
+	f4->DS4DEVF2 = 0x40; /* allow format 8 and 9 labels */
+	bzero(&f4->res5, sizeof(f4->res5));
 }
 
 
@@ -652,16 +724,15 @@ vtoc_init_format7_label (format7_label_t *f7)
 
 
 /*
- * initializes a format1 label
+ * helper function that initializes a most parts of a
+ * format1 or format 8 label, all but the field DS1FMTID
  */
-void 
-vtoc_init_format1_label (
-	char *volid,
+static void vtoc_init_format_1_8_label (
         unsigned int blksize,
 	extent_t *part_extent,
-	format1_label_t *f1) 
+	format1_label_t *f1)
 {
-        struct tm * creatime;
+	struct tm * creatime;
 	time_t t;
 	char str[80];
 
@@ -672,54 +743,86 @@ vtoc_init_format1_label (
 	bzero(f1->DS1DSNAM, sizeof(f1->DS1DSNAM));
 	sprintf(str, "PART    .NEW                                ");
 	vtoc_ebcdic_enc(str, str, 44);
-	strncpy(f1->DS1DSNAM, str, 44);
-	f1->DS1FMTID = 0xf1;
-	strncpy(f1->DS1DSSN, "      ", 6);
+	memcpy(f1->DS1DSNAM, str, 44);
+	memcpy((char *) f1->DS1DSSN, "      ", 6);
 	f1->DS1VOLSQ = 0x0001;
 
 	vtoc_set_date(&f1->DS1CREDT,
-                      (u_int8_t) creatime->tm_year,
+		      (u_int8_t) creatime->tm_year,
 		      (u_int16_t) creatime->tm_yday);
 	/* expires never - 99 365 */
-        vtoc_set_date(&f1->DS1EXPDT,
+	vtoc_set_date(&f1->DS1EXPDT,
 	              0x63,
-                      0x016D);
+		      0x016D);
 	f1->DS1NOEPV = 0x01;
 	f1->DS1NOBDB = 0x00;
-        f1->DS1FLAG1 = 0x00;
+	f1->DS1FLAG1 = 0x00;
 	vtoc_ebcdic_enc("IBM LINUX    ", str, 13);
-	strncpy(f1->DS1SYSCD, str, 13);
-        vtoc_set_date(&f1->DS1REFD,
+	memcpy((char *)f1->DS1SYSCD, str, 13);
+	vtoc_set_date(&f1->DS1REFD,
 		      (u_int8_t) creatime->tm_year,
-                      (u_int16_t) creatime->tm_yday);
-        f1->DS1SMSFG = 0x00;
-        f1->DS1SCXTF = 0x00;
-        f1->DS1SCXTV = 0x0000;
-        f1->DS1DSRG1 = 0x00;
-        f1->DS1DSRG2 = 0x00;
-        f1->DS1RECFM = 0x88;
-        f1->DS1OPTCD = 0x00;
-        f1->DS1BLKL  = blksize;
-        f1->DS1LRECL = blksize;
-        f1->DS1KEYL  = 0x00;
-        f1->DS1RKP   = 0x0000;
-        f1->DS1DSIND = 0x80; /* last volume for this dataset */
-        f1->DS1SCAL1 = 0x80;
-        bzero(&f1->DS1SCAL3, sizeof(f1->DS1SCAL3));
+		      (u_int16_t) creatime->tm_yday);
+	f1->DS1SMSFG = 0x00;
+	f1->DS1SCXTF = 0x00;
+	f1->DS1SCXTV = 0x0000;
+	f1->DS1DSRG1 = 0x00;
+	f1->DS1DSRG2 = 0x00;
+	f1->DS1RECFM = 0x88;
+	f1->DS1OPTCD = 0x00;
+	f1->DS1BLKL  = blksize;
+	f1->DS1LRECL = blksize;
+	f1->DS1KEYL  = 0x00;
+	f1->DS1RKP   = 0x0000;
+	f1->DS1DSIND = 0x80; /* last volume for this dataset */
+	f1->DS1SCAL1 = 0x80;
+	bzero(&f1->DS1SCAL3, sizeof(f1->DS1SCAL3));
 	vtoc_set_ttr(&f1->DS1LSTAR,
-	             0x0000,
-	             0x00);
+		     0x0000,
+		     0x00);
 	f1->DS1TRBAL = 0x00;
 	bzero(&f1->res1, sizeof(f1->res1));
-        memcpy(&f1->DS1EXT1,
-               part_extent,
-               sizeof(extent_t));
-        bzero(&f1->DS1EXT2, sizeof(extent_t));
-        bzero(&f1->DS1EXT3, sizeof(extent_t));
-        vtoc_set_cchhb(&f1->DS1PTRDS,
-                       0x0000, 0x0000, 0x00);
+	memcpy(&f1->DS1EXT1,
+	       part_extent,
+	       sizeof(extent_t));
+	bzero(&f1->DS1EXT2, sizeof(extent_t));
+	bzero(&f1->DS1EXT3, sizeof(extent_t));
+	vtoc_set_cchhb(&f1->DS1PTRDS,
+		       0x0000, 0x0000, 0x00);
 }
 
+void vtoc_init_format1_label (
+        unsigned int blksize,
+	extent_t *part_extent,
+	format1_label_t *f1)
+{
+	vtoc_init_format_1_8_label(blksize, part_extent, f1);
+	f1->DS1FMTID = 0xf1;
+}
+
+void vtoc_init_format8_label (
+        unsigned int blksize,
+	extent_t *part_extent,
+	format1_label_t *f8)
+{
+	vtoc_init_format_1_8_label(blksize, part_extent, f8);
+	f8->DS1FMTID = 0xf8;
+}
+
+void vtoc_update_format8_label (
+	cchhb_t *associated_f9,
+	format1_label_t *f8)
+{
+	memcpy(&f8->DS1PTRDS, associated_f9, sizeof(*associated_f9));
+}
+
+void vtoc_init_format9_label (
+	format9_label_t *f9)
+{
+	f9->DS9KEYID = 0x09;
+	f9->DS9SUBTY = 0x01;
+	f9->DS9NUMF9 = 1;
+	f9->DS9FMTID = 0xf9;
+}
 
 /*
  * do some updates to the VTOC format4 label
@@ -789,7 +892,6 @@ vtoc_reorganize_FMT5_extents (format5_label_t *f5)
 void 
 vtoc_update_format5_label_add (format5_label_t *f5,
 			       int verbose,
-			       int cyl,
 			       int trk,
 			       u_int16_t a, 
 			       u_int16_t b, 
@@ -885,7 +987,6 @@ vtoc_update_format5_label_add (format5_label_t *f5,
 void 
 vtoc_update_format5_label_del (format5_label_t *f5,
 			       int verbose,
-			       int cyl,
 			       int trk,
 			       u_int16_t a,
 			       u_int16_t b,
@@ -968,8 +1069,8 @@ vtoc_update_format5_label_del (format5_label_t *f5,
 			ext->fc = (a - ext->t) / trk;
 			ext->ft = (a - ext->t) % trk;
 
-			vtoc_update_format5_label_add(f5, verbose, 
-						      cyl, trk, x, y, z);
+			vtoc_update_format5_label_add(f5, verbose,
+						      trk, x, y, z);
 
 			if (verbose) 
 				printf("FMT5 del extent: 2 pieces\n");
@@ -1090,7 +1191,7 @@ void vtoc_update_format7_label_add (format7_label_t *f7, int verbose,
 		if ((ext->a + ext->b) == 0x00000000)
 			continue;
 
-		if ((ext->b + 1) == tmp->a) {
+		if ((ext->b) == tmp->a) {
 			/* this extent precedes the new one */
 			ext->b = tmp->b;
 			bzero(tmp, sizeof(ds7ext_t));
@@ -1102,7 +1203,7 @@ void vtoc_update_format7_label_add (format7_label_t *f7, int verbose,
 			continue;
 		} 
 
-		if (ext->a == (tmp->b + 1))
+		if (ext->a == (tmp->b))
 		{
 			/* this extent succeeds the new one */
 			ext->a = tmp->a;
@@ -1126,7 +1227,7 @@ void vtoc_update_format7_label_del (format7_label_t *f7, int verbose,
 {
 	ds7ext_t *ext;
 	int i, counter=0;
-	
+
 	for (i=0; i<16; i++) {
 		if (i<5) 
 			ext = &f7->DS7EXTNT[i]; 
@@ -1144,7 +1245,7 @@ void vtoc_update_format7_label_del (format7_label_t *f7, int verbose,
 
 		if ((a == ext->a) && (b < ext->b)) {
 			/* left-bounded in free space gap */
-			ext->a = b + 1;
+			ext->a = b;
 			if (verbose) 
 				printf("FMT7 add extent: left-bounded\n");
 			counter++;
@@ -1153,7 +1254,7 @@ void vtoc_update_format7_label_del (format7_label_t *f7, int verbose,
 
 		if ((a > ext->a) && (b == ext->b)) {
 			/* right-bounded in free space gap */
-			ext->b = a - 1;
+			ext->b = a;
 			if (verbose) 
 				printf("FMT7 add extent: right-bounded\n");
 			counter++;
@@ -1163,8 +1264,8 @@ void vtoc_update_format7_label_del (format7_label_t *f7, int verbose,
 		if ((a > ext->a) && (b < ext->b)) {
 			/* partition devides free space into 2 pieces */
 			vtoc_update_format7_label_add(f7, verbose, 
-						      b+1, ext->b);
-			ext->b = a - 1;
+						      b, ext->b);
+			ext->b = a;
 			if (verbose) 
 				printf("FMT7 add extent: 2 pieces\n");
 			counter++;
@@ -1193,14 +1294,18 @@ void vtoc_update_format7_label_del (format7_label_t *f7, int verbose,
  */
 void vtoc_set_freespace(format4_label_t *f4, format5_label_t *f5,
 		   format7_label_t *f7, char ch, int verbose,
-		   u_int32_t start, u_int32_t stop, int cyl, int trk)
+		   u_int32_t start, u_int32_t stop, u_int32_t cyl, u_int32_t trk)
 {
 	if ((cyl * trk) > BIG_DISK_SIZE) {
 		if (ch == '+') {
-			vtoc_update_format7_label_add(f7, verbose, start,stop);
+			vtoc_update_format7_label_add(f7, verbose, start,
+						      /* ds7ext RTA + 1 */
+						      stop + 1);
 		}
 		else if (ch == '-') {
-			vtoc_update_format7_label_del(f7, verbose, start,stop);
+			vtoc_update_format7_label_del(f7, verbose, start,
+						      /* ds7ext RTA + 1 */
+						      stop + 1);
 		}
 		else {
 			printf("BUG: syntax error in vtoc_set_freespace.\n");
@@ -1211,8 +1316,7 @@ void vtoc_set_freespace(format4_label_t *f4, format5_label_t *f5,
 		f4->DS4VTOCI = 0xa0;
 		f4->DS4EFLVL = 0x07;
 		vtoc_set_cchhb(&f4->DS4EFPTR, 0x0000, 0x0001, 0x03);
-	}
-	else {
+	} else {
 		u_int16_t x,y;
 		u_int8_t  z;
 
@@ -1221,11 +1325,11 @@ void vtoc_set_freespace(format4_label_t *f4, format5_label_t *f5,
 		z =  (u_int8_t) ((stop - start + 1) % trk);
 
 		if (ch == '+') {
-			vtoc_update_format5_label_add(f5, verbose, cyl, trk,
+			vtoc_update_format5_label_add(f5, verbose, trk,
 						      x, y, z);
 		}
 		else if (ch == '-') {
-			vtoc_update_format5_label_del(f5, verbose, cyl, trk,
+			vtoc_update_format5_label_del(f5, verbose, trk,
 						      x, y, z);
 		}
 		else {
