@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: MIT
 //
-// Copyright IBM Corp. 2023
+// Copyright IBM Corp. 2023, 2024
 
 use std::{
     io::{Read, Seek, SeekFrom::Current},
@@ -8,9 +8,8 @@ use std::{
 };
 
 // (SE) boot request control block aka SE header
-use crate::{assert_size, static_assert, Error, Result, PAGESIZE};
+use crate::{assert_size, request::MagicValue, static_assert, Error, Result, PAGESIZE};
 use log::debug;
-use pv_core::request::MagicValue;
 use zerocopy::{AsBytes, BigEndian, FromBytes, FromZeroes, U32, U64};
 
 /// Struct containing all SE-header tags.
@@ -19,43 +18,55 @@ use zerocopy::{AsBytes, BigEndian, FromBytes, FromZeroes, U32, U64};
 /// Page List Digest (pld)
 /// Address List Digest (ald)
 /// Tweak List Digest (tld)
-/// SE Header Tag (seht)
-///
+/// SE-Header Tag (tag)
 #[repr(C)]
-#[derive(Debug, Clone, Copy, AsBytes, PartialEq, Eq)]
+#[derive(Debug, Clone, Copy, AsBytes, PartialEq, Eq, FromBytes, FromZeroes)]
 pub struct BootHdrTags {
     pld: [u8; BootHdrHead::DIGEST_SIZE],
     ald: [u8; BootHdrHead::DIGEST_SIZE],
     tld: [u8; BootHdrHead::DIGEST_SIZE],
-    seht: [u8; BootHdrHead::SEHT_SIZE],
+    tag: [u8; BootHdrHead::TAG_SIZE],
+}
+assert_size!(BootHdrTags, 0xd0);
+
+impl AsRef<[u8]> for BootHdrTags {
+    fn as_ref(&self) -> &[u8] {
+        self.as_bytes()
+    }
 }
 
-/// Magiv value for a SE-(boot)header
+impl TryFrom<Vec<u8>> for BootHdrTags {
+    type Error = Error;
+
+    fn try_from(value: Vec<u8>) -> Result<Self, Self::Error> {
+        Self::ref_from(&value)
+            .ok_or_else(|| Error::InvBootHdrSize(value.len()))
+            .copied()
+    }
+}
+
+/// Magic value for a SE-(boot)header
+#[derive(Debug)]
 pub struct BootHdrMagic;
 impl MagicValue<8> for BootHdrMagic {
     const MAGIC: [u8; 8] = [0x49, 0x42, 0x4d, 0x53, 0x65, 0x63, 0x45, 0x78];
 }
 
 impl BootHdrTags {
-    /// Returns a reference to the SE-hdr tag of this [`BootHdrTags`].
-    pub fn seht(&self) -> &[u8; 16] {
-        &self.seht
+    /// Returns a reference to the SE-header tag of this [`BootHdrTags`].
+    pub fn tag(&self) -> &[u8; 16] {
+        &self.tag
     }
 
     /// Creates a new [`BootHdrTags`]. Useful for writing tests.
     #[doc(hidden)]
-    pub const fn new(pld: [u8; 64], ald: [u8; 64], tld: [u8; 64], seht: [u8; 16]) -> Self {
-        Self {
-            ald,
-            tld,
-            pld,
-            seht,
-        }
+    pub const fn new(pld: [u8; 64], ald: [u8; 64], tld: [u8; 64], tag: [u8; 16]) -> Self {
+        Self { ald, tld, pld, tag }
     }
 
-    /// returns false if no hdr found, true otherwise
-    /// in the very unlikel case an IO error can appear
-    /// when seeking to the beginning of  the header
+    /// Returns `false` if no SE-header found, `true` otherwise.
+    /// In the very unlikely case an IO error can appear
+    /// when seeking to the beginning of the header.
     fn seek_se_hdr_start<R>(img: &mut R) -> Result<bool>
     where
         R: Read + Seek,
@@ -93,8 +104,8 @@ impl BootHdrTags {
     ///
     /// # Errors
     ///
-    /// This function will return an error if `hdr` is not at least as long as the header specifies
-    /// in bytes 12-15 or the first 8 bytes do not contain the magic value.
+    /// This function will return an error if the header could not be found in
+    /// `img` or is invalid.
     pub fn from_se_image<R>(img: &mut R) -> Result<Self>
     where
         R: Read + Seek,
@@ -107,36 +118,42 @@ impl BootHdrTags {
         let mut hdr = vec![0u8; size_of::<BootHdrHead>()];
         img.read_exact(&mut hdr)?;
 
+        // Very unlikely - seek_se_hdr_start should point to a header or error-out
+        if !BootHdrMagic::starts_with_magic(&hdr) {
+            debug!("Inv magic");
+            return Err(Error::InvBootHdr);
+        }
+
         let hdr_head = match BootHdrHead::read_from_prefix(hdr.as_mut_slice()) {
             Some(hdr) => hdr,
             None => {
-                debug!("Boot hdr is to small");
+                debug!("Boot hdr is too small");
                 return Err(Error::InvBootHdr);
             }
         };
 
-        //Some sanity checks
-        if !BootHdrMagic::starts_with_magic(&hdr) || hdr_head.version.get() != 0x100 {
-            debug!("Inv magic or size");
+        // Some sanity checks
+        if hdr_head.version.get() != 0x100 {
+            debug!("Unsupported hdr-version: {:0>4x}", hdr_head.version.get());
             return Err(Error::InvBootHdr);
         }
 
-        //go to the Bot header tag
+        // go to the Boot header tag
         img.seek(Current(
             hdr_head.size.get() as i64
                 - size_of::<BootHdrHead>() as i64
-                - BootHdrHead::SEHT_SIZE as i64,
+                - BootHdrHead::TAG_SIZE as i64,
         ))?;
 
         // read in the tag
-        let mut seht = [0u8; BootHdrHead::SEHT_SIZE];
-        img.read_exact(seht.as_mut_slice())?;
+        let mut tag = [0u8; BootHdrHead::TAG_SIZE];
+        img.read_exact(tag.as_mut_slice())?;
 
         Ok(BootHdrTags {
             pld: hdr_head.pld,
             ald: hdr_head.ald,
             tld: hdr_head.tld,
-            seht,
+            tag,
         })
     }
 }
@@ -161,7 +178,7 @@ struct BootHdrHead {
 assert_size!(BootHdrHead, 0x1A0);
 impl BootHdrHead {
     const DIGEST_SIZE: usize = 0x40;
-    const SEHT_SIZE: usize = 0x10;
+    const TAG_SIZE: usize = 0x10;
 }
 
 #[cfg(test)]
@@ -194,7 +211,7 @@ mod tests {
             0x8f, 0x9b, 0xe0, 0xa5, 0x49, 0xd8, 0xd7, 0xa9, 0x4a, 0xe7, 0x20, 0xe5, 0xc0, 0x76,
             0x0a, 0x82, 0x5d, 0x47, 0x9f, 0xe6, 0x7a, 0xf5,
         ],
-        seht: [
+        tag: [
             0x92, 0x30, 0x9d, 0x45, 0x89, 0xb9, 0xa8, 0x5b, 0x42, 0x7f, 0x87, 0x53, 0x17, 0x1d,
             0x15, 0x20,
         ],
@@ -225,7 +242,7 @@ mod tests {
             Err(Error::InvBootHdr)
         ));
 
-        //header is at a non expected position
+        // header is at a non expected position
         let mut img = vec![0u8; PAGESIZE];
         img[0x008..0x288].copy_from_slice(bin_hdr);
         assert!(matches!(
@@ -242,4 +259,24 @@ mod tests {
         let hdr_tags = BootHdrTags::from_se_image(&mut Cursor::new(img)).unwrap();
         assert_eq!(hdr_tags, EXP_HDR);
     }
+
+    #[test]
+    fn tags_convert_u8() {
+        let bin_hdr = get_test_asset!("exp/secure_guest.hdr");
+        let hdr_tags = BootHdrTags::from_se_image(&mut Cursor::new(*bin_hdr)).unwrap();
+        let ser: &[u8] = hdr_tags.as_ref();
+        let mut ser = ser.to_vec();
+
+        let der: BootHdrTags = ser.clone().try_into().unwrap();
+        assert_eq!(hdr_tags, der);
+
+        ser.pop();
+        let der: Result<BootHdrTags> = ser.clone().try_into();
+        assert!(matches!(der, Err(Error::InvBootHdrSize(_))));
+
+        ser.push(17);
+        ser.push(17);
+        let der: Result<BootHdrTags> = ser.clone().try_into();
+        assert!(matches!(der, Err(Error::InvBootHdrSize(_))));
+    }
 }
