@@ -1,6 +1,10 @@
 // SPDX-License-Identifier: MIT
 //
-// Copyright IBM Corp. 2023
+// Copyright IBM Corp. 2023, 2024
+
+use std::path::PathBuf;
+
+use crate::secret::UserDataType;
 
 /// Result type for this crate
 pub type Result<T, E = Error> = std::result::Result<T, E>;
@@ -24,7 +28,7 @@ pub enum Error {
 
     #[error("Cannot load {ty}  from {path}")]
     X509Load {
-        path: String,
+        path: PathBuf,
         ty: &'static str,
         source: openssl::error::ErrorStack,
     },
@@ -41,13 +45,16 @@ pub enum Error {
     #[error("Provided binary request is too small")]
     BinRequestSmall,
 
-    #[error("No Config UID found: {0}")]
+    #[error("No Configuration UID found: {0}")]
     NoCuid(String),
 
     // errors from request types
     #[error("Customer Communication Key must be 32 bytes long")]
     CckSize,
 
+    #[error("Decryption failed. Probably due to a GCM tag mismatch.")]
+    GcmTagMismatch,
+
     #[error("Invalid {0} user-data for signing provided. Max {} bytes allowed", .0.max())]
     AsrcbInvSgnUserData(UserDataType),
 
@@ -68,6 +75,37 @@ pub enum Error {
     )]
     AsrcbUserDataSgnFail,
 
+    #[error("The provided Host Key Document in '{hkd}' is not in PEM or DER format")]
+    HkdNotPemOrDer {
+        hkd: String,
+        source: openssl::error::ErrorStack,
+    },
+
+    #[error("The provided host key document in {0} contains no certificate!")]
+    NoHkdInFile(String),
+
+    #[error("Invalid input size ({0}) for boot hdr")]
+    InvBootHdrSize(usize),
+
+    #[error("Input does not contain an attestation request")]
+    NoArcb,
+
+    #[error("The attestation request has an unknown version (.0)")]
+    BinArcbInvVersion(u32),
+
+    #[error(
+        "The attestation request encrypted sice is to0 small (.0). Request probably tampered with."
+    )]
+    BinArcbSeaSmall(u32),
+
+    #[error("The input is missing the Configuration UID entry. It is probably not an attestation response")]
+    AttExCuidMissing,
+
+    #[error(
+        "Attestation flags indicating that the additional data contains {0}, but no data was provided."
+    )]
+    AddDataMissing(&'static str),
+
     // errors from other crates
     #[error(transparent)]
     PvCore(#[from] pv_core::Error),
@@ -82,12 +120,11 @@ pub enum Error {
 // used in macros
 #[doc(hidden)]
 impl Error {
-    pub const CRL: &'static str = "CRL";
     pub const CERT: &'static str = "certificate";
+    pub const CRL: &'static str = "CRL";
 }
 
 /// Error cases for verifying host-key documents
-///
 #[allow(missing_docs)]
 #[derive(thiserror::Error, Debug, PartialEq, Eq)]
 #[non_exhaustive]
@@ -97,7 +134,7 @@ pub enum HkdVerifyErrorType {
     #[error("No valid CRL found")]
     NoCrl,
     #[error("Host-key document is revoked.")]
-    HdkRevoked,
+    HkdRevoked,
     #[error("Not enough bits of security. ({0}, {1} expected)")]
     SecurityBits(u32, u32),
     #[error("Authority Key Id mismatch")]
@@ -126,5 +163,3 @@ macro_rules! bail_hkd_verify {
     };
 }
 pub(crate) use bail_hkd_verify;
-
-use crate::request::uvsecret::UserDataType;
