@@ -2,9 +2,13 @@
 //
 // Copyright IBM Corp. 2023
 
-use crate::crypto::AES_256_GCM_TAG_SIZE;
+use crate::assert_size;
+use crate::crypto::{
+    decrypt_aes_gcm, derive_key, encrypt_aes_gcm, gen_ec_key, random_array, AesGcmResult, SymKey,
+    SymKeyType, AES_256_GCM_TAG_SIZE,
+};
 use crate::misc::to_u32;
-use crate::request::{derive_key, encrypt_aes_gcm, gen_ec_key, random_array, SymKey, SymKeyType};
+use crate::request::Confidential;
 use crate::{Error, Result};
 use openssl::bn::{BigNum, BigNumContext};
 use openssl::ec::{EcGroupRef, EcPointRef};
@@ -14,8 +18,6 @@ use openssl::pkey::{PKey, PKeyRef, Private, Public};
 use pv_core::request::{RequestMagic, RequestVersion};
 use std::convert::TryInto;
 use std::mem::size_of;
-use std::ops::Range;
-use utils::assert_size;
 use zerocopy::{AsBytes, BigEndian, FromBytes, FromZeroes, U32};
 
 /// Encrypt a _secret_ using self and a given private key.
@@ -28,7 +30,7 @@ pub trait Encrypt {
     /// # Errors
     ///
     /// This function will return an error if OpenSSL could not encrypt the secret.
-    fn encrypt(&self, secret: &[u8], priv_key: &PKey<Private>) -> Result<Vec<u8>> {
+    fn encrypt(&self, secret: &[u8], priv_key: &PKeyRef<Private>) -> Result<Vec<u8>> {
         let mut res = Vec::with_capacity(80);
         self.encrypt_to(secret, priv_key, &mut res)?;
         Ok(res)
@@ -43,10 +45,16 @@ pub trait Encrypt {
     /// # Errors
     ///
     /// This function will return an error if OpenSSL could not encrypt the secret.
-    fn encrypt_to(&self, secret: &[u8], priv_key: &PKey<Private>, to: &mut Vec<u8>) -> Result<()>;
+    fn encrypt_to(
+        &self,
+        secret: &[u8],
+        priv_key: &PKeyRef<Private>,
+        to: &mut Vec<u8>,
+    ) -> Result<()>;
 }
 
 /// Types of Authenticated Data
+#[allow(missing_debug_implementations)]
 pub enum Aad<'a> {
     /// Authenticated Keyslot
     Ks(&'a Keyslot),
@@ -82,7 +90,9 @@ impl Keyslot {
 impl Encrypt for Keyslot {
     /// Encrypts the given request protection key `prot_key`.
     ///
-    /// The AES256 encryption key is derived from `self` as public key, and `priv_key` as private key.
+    /// The AES256 encryption key is derived from `self` as public key, and `priv_key` as private
+    /// key.
+    ///
     /// # Returns
     /// The encrypted Keyslot.
     ///
@@ -92,11 +102,12 @@ impl Encrypt for Keyslot {
     fn encrypt_to(
         &self,
         prot_key: &[u8],
-        priv_key: &PKey<Private>,
+        priv_key: &PKeyRef<Private>,
         to: &mut Vec<u8>,
     ) -> Result<()> {
         let derived_key = derive_key(priv_key, &self.0)?;
-        let (mut wrpk_and_kst, ..) = encrypt_aes_gcm(&derived_key.into(), &[0; 12], &[], prot_key)?;
+        let mut wrpk_and_kst =
+            encrypt_aes_gcm(&derived_key.into(), &[0; 12], &[], prot_key)?.data();
         let phk: EcdhPubkeyCoord = self.0.as_ref().try_into()?;
 
         to.reserve(80);
@@ -106,7 +117,7 @@ impl Encrypt for Keyslot {
     }
 }
 
-/// Context used to mange the encryption of requests.
+/// Context used to manage the encryption of requests.
 /// Intended to be used by [`Request`] implementations
 #[derive(Debug)]
 pub struct ReqEncrCtx {
@@ -143,7 +154,7 @@ impl ReqEncrCtx {
             prot_key,
         })
     }
-    ///
+
     /// Create a new encryption context with random input values.
     ///
     /// # Errors
@@ -155,12 +166,6 @@ impl ReqEncrCtx {
         }
     }
 
-    ///Panics if data does not fit into bin_aad+offs
-    // #[track_caller]
-    // pub fn copy_to_bin_aad(_bin_aad: &mut [u8], _aad_offs: usize, _data: &[u8]) {
-    //     todo!();
-    // }
-
     /// Build the authenticated data for a request.
     /// # Returns
     /// ```none
@@ -172,7 +177,6 @@ impl ReqEncrCtx {
     /// |                Request type dependent AAD data              |
     /// |-------------------------------------------------------------|
     /// ```
-    ///
     pub fn build_aad<O>(
         &self,
         version: RequestVersion,
@@ -202,8 +206,8 @@ impl ReqEncrCtx {
         }?;
         let mut auth_data: Vec<u8> = Vec::with_capacity(2048);
 
-        //reserve space for the request header
-        auth_data.resize(std::mem::size_of::<RequestHdr>(), 0);
+        // reserve space for the request header
+        auth_data.resize(size_of::<RequestHdr>(), 0);
 
         for a in aad {
             match a {
@@ -225,11 +229,11 @@ impl ReqEncrCtx {
 
         let req_hdr = RequestHdr::new(version, rql, self.iv, nks, sea, magic);
         // copy request header to the start of the request
-        auth_data[..std::mem::size_of::<RequestHdr>()].copy_from_slice(req_hdr.as_bytes());
+        auth_data[..size_of::<RequestHdr>()].copy_from_slice(req_hdr.as_bytes());
         Ok(auth_data)
     }
 
-    /// get the public coordinates from the private key (Customer private key)
+    /// Get the public coordinates from the private key (Customer private key)
     /// # Errors
     ///
     /// This function will return an error if the public key could not be extracted by OpenSSL.
@@ -238,7 +242,7 @@ impl ReqEncrCtx {
         self.priv_key.as_ref().try_into().map_err(Error::Crypto)
     }
 
-    /// Encrypt confidential Data with this encryption context and provide a gcm tag.
+    /// Encrypt confidential Data with this encryption context and provide a GCM tag.
     ///
     /// * `aad` - additional authentic data
     /// * `conf` - data to be encrypted
@@ -252,13 +256,14 @@ impl ReqEncrCtx {
     /// # Errors
     ///
     /// This function will return an error if the data could not be encrypted by OpenSSL.
-    pub fn encrypt_aead(
-        &self,
-        aad: &[u8],
-        conf: &[u8],
-    ) -> Result<(Vec<u8>, Range<usize>, Range<usize>, Range<usize>)> {
+    pub(crate) fn encrypt_aead(&self, aad: &[u8], conf: &[u8]) -> Result<AesGcmResult> {
         encrypt_aes_gcm(&self.prot_key, &self.iv, aad, conf)
     }
+
+    /// Returns a reference to the request protection key of this [`ReqEncrCtx`].
+    pub fn prot_key(&self) -> &SymKey {
+        &self.prot_key
+    }
 }
 
 #[repr(C)]
@@ -270,8 +275,8 @@ impl AsRef<[u8]> for EcdhPubkeyCoord {
     }
 }
 
-/// Get the pub ecdh coordinates in the format the Ultravisor expects it:
-/// The two coordinates are pdadded to 80 bytes each.
+/// Get the pub ECDH coordinates in the format the Ultravisor expects it:
+/// The two coordinates are padded to 80 bytes each.
 fn get_pub_ecdh_points(pkey: &EcPointRef, grp: &EcGroupRef) -> Result<[u8; 160], ErrorStack> {
     const ECDH_PUB_KEY_COORD_POINT_SIZE: i32 = 0x50;
     let mut x = BigNum::new()?;
@@ -287,6 +292,7 @@ macro_rules! ecdh_from {
     ($type: ty) => {
         impl TryFrom<&PKeyRef<$type>> for EcdhPubkeyCoord {
             type Error = ErrorStack;
+
             fn try_from(key: &PKeyRef<$type>) -> Result<Self, Self::Error> {
                 let k = key.ec_key()?;
                 k.check_key()?;
@@ -345,9 +351,9 @@ impl RequestHdr {
 /// `ReqEncrCtx` when implementing `encrypt`. A hostkey should be represented by [`Keyslot`] during
 /// encryption.
 ///
-/// An UV request consists of an authenticated area (AAD), an encrypted area (Encr) and a 16 byte tag.
-/// The AAD contains a general header and Request type defined data (including Keyslots).
-/// It is encrypted with an Request protection key (symmetric). This key is encrypted with a
+/// An UV request consists of an authenticated area (AAD), an encrypted area (Encr) and a 16 byte
+/// tag. The AAD contains a general header and Request type defined data (including Keyslots). It
+/// is encrypted with an Request protection key (symmetric). This key is encrypted with a
 /// (generated) private key and the public key of the host system (Host key)
 /// ```none
 ///  _______________________________________________________________
@@ -361,7 +367,7 @@ impl RequestHdr {
 ///  |     ----------------------------------------------------    |
 ///  |                   AES GCM Tag (16)                          |
 ///  |_____________________________________________________________|
-///```
+/// ```
 pub trait Request {
     /// Encrypt the request into its binary format
     ///
@@ -378,7 +384,6 @@ pub trait Request {
 
 /// A struct to represent some parts of a binary/encrypted request.
 #[derive(Debug)]
-#[allow(unused)]
 #[allow(clippy::len_without_is_empty)]
 pub struct BinReqValues<'a> {
     iv: &'a [u8],
@@ -435,6 +440,28 @@ impl<'a> BinReqValues<'a> {
     pub fn len(&self) -> usize {
         self.len
     }
+
+    /// Returns the size of the encrypted area
+    pub fn sea(&self) -> u32 {
+        self.encr.len() as u32
+    }
+
+    /// Decrypts the encrypted area with the provided key
+    pub fn decrypt(&self, key: &SymKey) -> Result<Confidential<Vec<u8>>> {
+        decrypt_aes_gcm(key, self.iv, self.aad, self.encr, self.tag)
+    }
+
+    /// Returns a reference to the request dependent authenticated area of this [`BinReqValues`]
+    /// already interpreted.
+    ///
+    /// If target struct is larger than the request depended-AAD None is returned. See
+    /// [`FromBytes::ref_from_prefix`]
+    pub fn req_dep_aad<T>(&self) -> Option<&T>
+    where
+        T: FromBytes + Sized,
+    {
+        T::ref_from_prefix(self.req_dep_aad)
+    }
 }
 
 #[cfg(test)]
@@ -465,15 +492,15 @@ mod tests {
             .unwrap();
 
         let mut aad_exp = vec![
-            0x12, 0x34, 0x56, 0x89, 0xab, 0xcd, 0xef, 0, //progr
+            0x12, 0x34, 0x56, 0x89, 0xab, 0xcd, 0xef, 0, // progr
             0, 0, 2, 0, // vers
-            0, 0, 0, 168, //size
+            0, 0, 0, 168, // size
             0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, // iv
-            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, //res
-            1, //nks
+            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // res
+            1, // nks
             0, 0, 0, 0, // res
             0, 0, 0, 16, // sea
-            0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, //aad
+            0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, // aad
         ];
         aad_exp.extend_from_slice(get_test_asset!("exp/keyslot.bin"));
         assert_eq!(&aad, &aad_exp);
@@ -506,7 +533,7 @@ mod tests {
         let (_, host_key) = get_test_keys();
         let ctx = ReqEncrCtx::new_aes_256(Some([0x11; 12]), None, None).unwrap();
 
-        let ks = vec![
+        let ks = [
             Keyslot::new(host_key.clone()),
             Keyslot::new(host_key.clone()),
             Keyslot::new(host_key),
@@ -524,12 +551,12 @@ mod tests {
         let hdr = RequestHdr::new(0x200, 22, [0x11; 12], 15, 44, None);
         let hdr_bin = hdr.as_bytes();
         let hdr_bin_exp = [
-            0u8, 0, 0, 0, 0, 0, 0, 0, //magic
+            0u8, 0, 0, 0, 0, 0, 0, 0, // magic
             0, 0, 2, 0, // vers
-            0, 0, 0, 22, //size
+            0, 0, 0, 22, // size
             0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, // iv
-            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  //res
-            15, //nks
+            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // res
+            15, // nks
             0, 0, 0, 0, // res
             0, 0, 0, 44, // sea
         ];
@@ -541,12 +568,12 @@ mod tests {
         let mut hdr = RequestHdr::new(0x200, 0x1234, [0x11; 12], 15, 44, Some(TEST_MAGIC));
         let hdr_bin = hdr.as_bytes_mut();
         let hdr_bin_exp = [
-            0x12, 0x34, 0x56, 0x89, 0xab, 0xcd, 0xef, 0, //magic
+            0x12, 0x34, 0x56, 0x89, 0xab, 0xcd, 0xef, 0, // magic
             0, 0, 2, 0, // vers
-            0, 0, 0x12, 0x34, //size
+            0, 0, 0x12, 0x34, // size
             0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, // iv
-            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  //res
-            15, //nks
+            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // res
+            15, // nks
             0, 0, 0, 0, // res
             0, 0, 0, 44, // sea
         ];
