@@ -1,23 +1,16 @@
+use crate::assert_size;
 use crate::{
     crypto::{sign_msg, verify_signature},
     req::BinReqValues,
     request::{
-        openssl::{
-            pkey::{PKey, Private},
-            MessageDigest,
-        },
-        uvsecret::{AddSecretRequest, AddSecretVersion},
+        openssl::pkey::{HasParams, HasPublic, Id, PKey, PKeyRef, Private, Public},
         RequestMagic,
     },
+    secret::{AddSecretMagic, AddSecretRequest, AddSecretVersion, UserDataType},
     Error, Result,
 };
-use openssl::{
-    nid::Nid,
-    pkey::{HasParams, HasPublic, Id, PKeyRef, Public},
-};
-use pv_core::request::uvsecret::AddSecretMagic;
-use pv_core::request::uvsecret::UserDataType;
-use utils::assert_size;
+use openssl::hash::MessageDigest;
+use openssl::nid::Nid;
 use zerocopy::{AsBytes, BigEndian, FromBytes, FromZeroes, U16};
 
 /// User data.
@@ -119,11 +112,12 @@ impl UserData {
         let magic: AddSecretMagic = self.data_type().into();
         magic.get()
     }
+
     /// Creates new user data
     ///
     /// Verifies that the provided data + signature fits into 512 bytes
     ///
-    /// #Error
+    /// # Error
     /// An error is reported if the provided data and the signature would not fit into 512 bytes
     /// An error is reported if the key is not of type RSA (2048|3072) or EC(specp521r1)
     pub(super) fn new(sign_key: Option<PKey<Private>>, data: Vec<u8>) -> Result<Self> {
@@ -252,7 +246,7 @@ fn check_key_format(kind: UserDataType, key: &PKeyRef<Public>) -> Result<()> {
 ///
 /// Extracrted user-data if available
 ///
-/// #Errors
+/// # Errors
 ///
 /// returns an error if
 /// - No sound add-secret request presented
@@ -271,7 +265,7 @@ pub fn verify_asrcb_and_get_user_data(
         return Err(Error::BinAsrcbInvVersion);
     }
 
-    //preventing the two lines after the truncate from panicking
+    // preventing the two lines after the truncate from panicking
     let req_len = req.len();
     if asrcb.len() < req_len
         || req_len < AddSecretRequest::V1_USER_DATA_OFFS + UserData::USER_DATA_SIZE
@@ -323,6 +317,7 @@ impl VerifiedUserData {
     /// Overwrites the signature in the buf with zeros.
     ///
     /// #Panics
+    ///
     /// Panics it provided buffer is smaller that 512 bytes or kind is Null or Unsigned
     fn new(buf: &mut [u8], kind: UserDataType) -> Self {
         assert!(buf.len() >= 0x200);
@@ -359,7 +354,7 @@ impl VerifiedUserData {
             UserDataType::Unsigned => unreachable!(),
         };
 
-        //overwrite signature field with zeros
+        // overwrite signature field with zeros
         sgn.fill(0);
         ret
     }
