@@ -18,7 +18,7 @@ use config::{ApConfigEntry, ApConfigList};
 use helper::{LockFile, PATH_PVAPCONFIG_LOCK};
 use pv_core::uv::{ListableSecretType, SecretList};
 use std::process::ExitCode;
-use utils::release_string;
+use utils::print_version;
 
 /// Simple macro for
 ///   if Cli::verbose() {
@@ -58,11 +58,7 @@ macro_rules! on_error_print_and_exit {
 fn main() -> ExitCode {
     // handle version option
     if cli::ARGS.version {
-        println!(
-            "{} version {}\nCopyright IBM Corp. 2023",
-            env!("CARGO_PKG_NAME"),
-            release_string!()
-        );
+        print_version!(0, "2023");
         return ExitCode::SUCCESS;
     }
 
@@ -528,8 +524,8 @@ mod tests {
     fn make_assoc_secretentry(idx: u16, hexidstr: &str) -> SecretEntry {
         let id = hexstring_to_u8(hexidstr);
         let idlen: u32 = id.len().try_into().unwrap();
-        let idarray = <&[u8; 32]>::try_from(id.as_slice()).unwrap();
-        SecretEntry::new(idx, ListableSecretType::Association, *idarray, idlen)
+        let idarray: [u8; 32] = id.try_into().unwrap();
+        SecretEntry::new(idx, ListableSecretType::Association, idarray.into(), idlen)
     }
 
     fn make_test_secrets() -> Vec<SecretEntry> {
@@ -607,13 +603,11 @@ mod tests {
     #[test]
     fn test_do_ap_config_invocation_1() {
         let test_apqns = make_test_apqns();
-        let mut apqns: Vec<Apqn> = Vec::new();
-        apqns.push(test_apqns[0].clone());
+        let apqns: Vec<Apqn> = vec![test_apqns[0].clone()];
         let secrets: Vec<SecretEntry> = Vec::new();
         let secretlist = SecretList::new(secrets.len() as u16, secrets);
         let test_apconfigs = make_test_apconfigs();
-        let mut apconfig: Vec<ApConfigEntry> = Vec::new();
-        apconfig.push(test_apconfigs[0].clone());
+        let apconfig: Vec<ApConfigEntry> = vec![test_apconfigs[0].clone()];
         let apcfglist = ApConfigList::from_apconfigentry_vec(apconfig);
         let mut apqnlist = ApqnList::from_apqn_vec(apqns);
         let r = do_ap_config(&mut apqnlist, &secretlist, &apcfglist, true);
@@ -625,16 +619,12 @@ mod tests {
     #[test]
     fn test_do_ap_config_invocation_2() {
         let test_apqns = make_test_apqns();
-        let mut apqns: Vec<Apqn> = Vec::new();
-        apqns.push(test_apqns[1].clone());
+        let apqns: Vec<Apqn> = vec![test_apqns[1].clone()];
         let mut secrets = make_test_secrets();
-        while secrets.len() > 2 {
-            secrets.pop();
-        }
+        secrets.truncate(2);
         let secretlist = SecretList::new(secrets.len() as u16, secrets);
         let test_apconfigs = make_test_apconfigs();
-        let mut apconfig: Vec<ApConfigEntry> = Vec::new();
-        apconfig.push(test_apconfigs[1].clone());
+        let apconfig: Vec<ApConfigEntry> = vec![test_apconfigs[1].clone()];
         let apcfglist = ApConfigList::from_apconfigentry_vec(apconfig);
         let mut apqnlist = ApqnList::from_apqn_vec(apqns);
         let r = do_ap_config(&mut apqnlist, &secretlist, &apcfglist, true);
