@@ -3,8 +3,7 @@
 // Copyright IBM Corp. 2023
 
 use clap::{ArgGroup, Args, CommandFactory, Parser, Subcommand, ValueEnum, ValueHint};
-use pv::misc::CertificateOptions;
-use pv::misc::STDOUT;
+use utils::{CertificateOptions, STDOUT};
 
 /// Manage secrets for IBM Secure Execution guests.
 ///
@@ -119,7 +118,7 @@ pub struct CreateSecretOpt {
     /// user data can be up to 512 bytes of arbitrary data, and the maximum size depends on the
     /// size of the user-signing key:
     /// - No key: user data can be 512 bytes.
-    /// - EC or RSA 2048 keys: user data can be 256 bytes.
+    /// - EC(secp521r1) or RSA 2048 keys: user data can be 256 bytes.
     /// - RSA 3072 key: user data can be 128 bytes.
     ///
     /// The firmware ignores this data, but the request tag protects the user-data. Optional. No
@@ -131,10 +130,11 @@ pub struct CreateSecretOpt {
     ///
     /// Adds a signature calculated from the key in <FILE> to the add-secret request. The
     /// file must be in DER or PEM format containing a private key. Supported are RSA 2048 &
-    /// 3072-bit and EC(secp521r1) keys. The firmware ignores the content, but the request tag protects the
-    /// signature. The user-signing key signs the request. The location of the signature is filled
-    /// with zeros during the signature calculation. The request tag also secures the signature.
-    /// See man pvsecret verify for more details. Optional. No signature by default.
+    /// 3072-bit and EC(secp521r1) keys. The firmware ignores the content, but the request tag
+    /// protects the signature. The user-signing key signs the request. The location of the
+    /// signature is filled with zeros during the signature calculation. The request tag also
+    /// secures the signature. See man pvsecret verify for more details. Optional. No signature
+    /// by default.
     #[arg(long, value_name = "FILE", value_hint = ValueHint::FilePath,)]
     pub user_sign_key: Option<String>,
 }
@@ -158,9 +158,9 @@ pub enum AddSecretType {
         /// white-spaces mapped to `_`.
         name: String,
 
-        ///Print the hashed name to stdout.
+        /// Print the hashed name to stdout.
         ///
-        ///The hashed name is not written to `NAME.yaml`
+        /// The hashed name is not written to `NAME.yaml`
         #[arg(long)]
         stdout: bool,
 
