@@ -346,11 +346,19 @@ int dfi_s390_init_gen(bool extended)
 	l.extended = extended;
 	if (read_s390_hdr() != 0)
 		return -ENODEV;
-	zg_ioctl(g.fh, BLKSSZGET, &l.blk_size, "BLKSSZGET", ZG_CHECK);
-	if (!extended)
+	if (!extended) {
 		rc = mem_chunks_add();
-	else
+	} else {
+		/*
+		 * A device block size is required for a decompression of
+		 * s390_ext dump with compressed dump segments.
+		 * Since dumps in s390_ext format can reside on DASD partition
+		 * only, bail out upon ioctl error on BLKSSZGET.
+		 */
+		if (zg_ioctl(g.fh, BLKSSZGET, &l.blk_size, "BLKSSZGET", ZG_CHECK_NONE))
+			return -ENODEV;
 		rc = mem_chunks_add_ext();
+	}
 	if (rc)
 		return rc;
 	rc = df_s390_cpu_info_add(&l.hdr, l.hdr.mem_size);
