@@ -53,16 +53,22 @@ typedef enum {
 	disk_type_eckd_cdl,
 } disk_type_t;
 
+struct disk_ext_type {
+	disk_type_t type;
+	bool is_nvme;
+};
+
 /* targetbase definition */
 typedef enum {
 	defined_as_device,
-	defined_as_name
+	defined_as_name,
+	undefined
 } definition_t;
 
 /* Disk information type */
 struct disk_info {
 	disk_type_t type;
-	dev_t device;
+	dev_t device; /* logical device for bootmap creation */
 	dev_t partition;
 	int devno;
 	int partnum;
@@ -72,8 +78,11 @@ struct disk_info {
 	struct hd_geometry geo;
 	char* name;
 	char* drv_name;
-	definition_t targetbase;
+	definition_t targetbase_def;
 	int is_nvme;
+	dev_t basedisks[MAX_TARGETS]; /* array of physical disks for
+				       * bootstrap blocks recording
+				       */
 };
 
 struct file_range {
@@ -83,12 +92,15 @@ struct file_range {
 
 struct job_target_data;
 
-int disk_get_info(const char* device, struct job_target_data* target,
-		  struct disk_info** info);
-int disk_is_tape(const char* device);
-int disk_is_scsi(const char* device, struct job_target_data* target);
-int disk_is_eckd_ldl(const char* device, struct job_target_data* target);
-int disk_is_nvme(const char* device, struct job_target_data* target);
+int disk_get_info(const char *device, struct job_target_data *target,
+		  struct disk_info **info);
+int disk_get_ext_type(const char *device, struct disk_ext_type *ext_type);
+int disk_is_tape(const char *device);
+int disk_type_is_scsi(struct disk_ext_type *ext_type);
+int disk_type_is_eckd_ldl(struct disk_ext_type *ext_type);
+int disk_type_is_nvme(struct disk_ext_type *ext_type);
+int disk_type_is_eckd(disk_type_t type);
+
 int disk_get_info_from_file(const char* filename,
 			    struct job_target_data* target,
 			    struct disk_info** info);
@@ -113,6 +125,9 @@ blocknum_t disk_write_block_buffer_align(int fd, int fd_is_basedisk,
 					 struct disk_info *info, int align,
 					 off_t *offset);
 void disk_print_devt(dev_t d);
+void disk_print_devname(dev_t d);
+void prepare_footnote_ptr(int source, char *ptr);
+void print_footnote_ref(int source, const char *prefix);
 void disk_print_info(struct disk_info *info, int source);
 int disk_is_zero_block(disk_blockptr_t* block, struct disk_info* info);
 blocknum_t disk_compact_blocklist(disk_blockptr_t* list, blocknum_t count,
@@ -122,7 +137,6 @@ blocknum_t disk_get_blocklist_from_file(const char* filename,
 					disk_blockptr_t** blocklist,
 					struct disk_info* pinfo);
 int disk_check_subchannel_set(int devno, dev_t device, char* dev_name);
-void disk_print_geo(struct disk_info *data);
 int fs_map(int fd, uint64_t offset, blocknum_t *mapped, int fs_block_size);
 
 #endif /* not DISK_H */
